@@ -7,8 +7,6 @@ import time
 import traceback
 import unittest
 
-from django.utils.unittest import result
-
 
 def trim_docstring(docstring):
     """Trim leading spaces in docstring indentation.
@@ -39,7 +37,7 @@ def trim_docstring(docstring):
     return '\n'.join(trimmed)
 
 
-class PipedTestResult(result.TestResult):
+class PipedTestResult(unittest.result.TestResult):
     """A test result class that can print test results in a machine-parseable format.
 
     Used by PipedTestRunner.
