@@ -804,7 +804,11 @@ class MainWindow(object):
 
 
 class StackTraceDialog(Toplevel):
-    def __init__(self, parent, title, label, trace, button_text='OK'):
+    OK = 1
+    CANCEL = 2
+
+    def __init__(self, parent, title, label, trace, button_text='OK',
+                 cancel_text='Cancel'):
         '''Show a dialog with a scrollable stack trace.
 
         Arguments:
@@ -814,6 +818,7 @@ class StackTraceDialog(Toplevel):
             label -- the label describing the stack trace
             trace -- the stack trace content to display.
             button_text -- the label for the button text ("OK" by default)
+            cancel_text -- the label for the cancel button ("Cancel" by default)
         '''
         Toplevel.__init__(self, parent)
 
@@ -836,17 +841,21 @@ class StackTraceDialog(Toplevel):
         self.label.grid(column=0, row=0, padx=5, pady=5, sticky=(W, E))
 
         self.description = ReadOnlyText(self.frame, width=80, height=20)
-        self.description.grid(column=0, row=1, pady=5, sticky=(N, S, E, W,))
+        self.description.grid(column=0, columnspan=2, row=1, pady=5, sticky=(N, S, E, W,))
 
         self.description_scrollbar = Scrollbar(self.frame, orient=VERTICAL)
-        self.description_scrollbar.grid(column=1, row=1, pady=5, sticky=(N, S))
+        self.description_scrollbar.grid(column=1, row=1, pady=5, sticky=(N, S, E))
         self.description.config(yscrollcommand=self.description_scrollbar.set)
         self.description_scrollbar.config(command=self.description.yview)
 
         self.description.insert('1.0', trace)
 
+        self.cancel_button = Button(self.frame, text=cancel_text, command=self.cancel)
+        self.cancel_button.grid(column=0, row=2, padx=5, pady=5, sticky=(E,))
+
         self.ok_button = Button(self.frame, text=button_text, command=self.ok, default=ACTIVE)
-        self.ok_button.grid(column=0, row=2, padx=5, pady=5, sticky=(E,))
+        self.ok_button.grid(column=1, row=2, padx=5, pady=5, sticky=(E,))
+
 
         self.columnconfigure(0, weight=1)
         self.rowconfigure(0, weight=1)
@@ -881,13 +890,17 @@ class StackTraceDialog(Toplevel):
         if self.parent is not None:
             self.parent.focus_set()
         self.destroy()
+        self.status = self.OK
 
     def cancel(self, event=None):
-        # put focus back to the parent window
+        self.withdraw()
+        self.update_idletasks()
+
         if self.parent is not None:
             self.parent.focus_set()
 
         self.destroy()
+        self.status = self.CANCEL
 
 
 class FailedTestDialog(StackTraceDialog):
@@ -905,8 +918,14 @@ class FailedTestDialog(StackTraceDialog):
             'Error running test suite',
             'The following stack trace was generated when attempting to run the test suite:',
             trace,
+            button_text='OK',
+            cancel_text='Quit',
         )
 
+    def cancel(self, event=None):
+        StackTraceDialog.cancel(self, event=event)
+        self.parent.quit()
+
 
 class TestLoadErrorDialog(StackTraceDialog):
     def __init__(self, parent, trace):
@@ -921,7 +940,35 @@ class TestLoadErrorDialog(StackTraceDialog):
             self,
             parent,
             'Error discovering test suite',
-            'The following stack trace was generated when attempting to discover the test suite:',
+            ('The following stack trace was generated when attempting to '
+             'discover the test suite:'),
             trace,
-            button_text='Retry'
+            button_text='Retry',
+            cancel_text='Quit',
+        )
+
+    def cancel(self, event=None):
+        StackTraceDialog.cancel(self, event=event)
+        self.parent.quit()
+
+
+class IgnorableTestLoadErrorDialog(StackTraceDialog):
+    def __init__(self, parent, trace):
+        '''Show a dialog with a scrollable stack trace when loading
+           tests turned up errors in stderr but they can safely be ignored.
+
+        Arguments:
+
+            parent -- a parent window (the application window)
+            trace -- the stack trace content to display.
+        '''
+        StackTraceDialog.__init__(
+            self,
+            parent,
+            'Error discovering test suite',
+            ('The following error where captured during test discovery '
+             'but running the tests might still work:'),
+            trace,
+            button_text='Continue',
+            cancel_text='Quit',
         )
